from django.test import TestCase, Client

# Create your tests here.
from .models import QuoteTag, Author, Work, Quote
import re
import lxml.etree

class QuoteTest(TestCase):
    def setUp(self):
        a1 = Author.objects.create(name="JFK")
        w1 = Work.objects.create(name="Berlin speech", author=a1)
        q1 = Quote.objects.create(text="<p>Ich bin...</p>", work=w1)
        self.q1 = q1

    def test_one(self):
        q = Quote.objects.filter(text__startswith="<p>Ich")
        self.assertEqual(q.count(), 1)
        q = q[0]
        self.assertEqual(q, self.q1)
        self.assertEqual(q.work.author.name, "JFK")

class ViewsTest(TestCase):
    def getPage(self, url, exp_status=200):
        c = Client()
        response = c.get('/quotes/' + url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.charset, 'utf-8')
        document = response.content.decode(response.charset)
        try:
            lxml.etree.fromstring(document)
        except lxml.etree.XMLSyntaxError as e:
            self.assertTrue(False, 'Errors in page at %s: %s' % (url, e))
        self.assertFalse('<script>' in document)
        return document

    def setUp(self):
        a1 = Author.objects.create(name="Author with notes",
                                   notes="<script>Some notes for the author</script>")
        w1 = Work.objects.create(name="Context with some notes",
                                 author=a1,
                                 notes="<p>Some notes for the work</p>")
        q1 = Quote.objects.create(text="<p>Quote01, two tags</p>",
                                  work=w1,
                                  notes="<p>Some notes for the quote</p>")
        t1 = QuoteTag.objects.create(tag="tag01-1")
        t2 = QuoteTag.objects.create(tag="tag01-2")
        q1.tags.add(t1, t2)

        a2 = Author.objects.create(name="Author without notes")
        w2 = Work.objects.create(name="Work without notes", author=a2)
        q2 = Quote.objects.create(text="<p>Quote02, no tags</p>", work=w2)
        self.assertSequenceEqual(q2.tags.all(), [])

    def test_all(self):
        content = self.getPage('all/')
        for a in Author.objects.all():
            self.assertIn(a.name, content)
            self.assertIn(a.notes, content)
        for w in Work.objects.all():
            self.assertIn(w.name, content)
            self.assertIn(w.notes, content)
        for q in Quote.objects.all():
            self.assertIn(q.text, content)
            self.assertIn(q.notes, content)

    def test_random(self):
        seen = {}
        for q in Quote.objects.all():
            seen[q.id] = False

        # <a href="{% url 'quotes:onequote' quote.id %}">Permalink</a>
        permalinkre = re.compile(r'([0-9]+).*Permalink')

        for i in range(100):
            content = self.getPage('random')
            m = permalinkre.search(content)
            self.assertIsNotNone(m, content)
            quoteid = int(m.group(1))
            self.assertIn(quoteid, seen)
            seen[quoteid] = True
        for q in Quote.objects.all():
            self.assertTrue(seen[q.id])

    def test_views_all_data(self):
        q = Quote.objects.filter(text__startswith="<p>Quote01")
        self.assertEqual(q.count(), 1, "Couldn't find Quote01 after insertion")
        q = q[0]

        # check the individual quote page; each of the note type is displayed
        quotepage = self.getPage('show/%s/' % q.id)
        self.assertIn(q.text, quotepage)
        self.assertIn("author_notes", quotepage)
        self.assertIn("work_notes", quotepage)
        self.assertIn("tag_link", quotepage)
        self.assertEqual(quotepage.count("tag_link"), 2)

        # check the work page; each of the note type is displayed, and
        # the work notes are shown only once
        workpage = self.getPage('work/%s/' % q.work.id)
        self.assertIn(q.text, workpage)
        self.assertIn("author_notes", workpage)
        self.assertIn("work_notes", workpage)
        self.assertEqual(workpage.count("work_notes"), 1)
        self.assertIn("tag_link", workpage)

        # check the author page; each of the note type is displayed, and
        # the author notes are shown only once
        authorpage = self.getPage('author/%s/' % q.work.author.id)
        self.assertIn(q.text, authorpage)
        self.assertIn("author_notes", authorpage)
        self.assertEqual(authorpage.count("author_notes"), 1)
        self.assertIn("work_notes", authorpage)
        self.assertIn("tag_link", authorpage)

        # check the tag page
        for tag in q.tags.all():
            tagpage = self.getPage('tag/%s/' % tag.id)
            self.assertIn(q.text, tagpage)

    def test_views_minimal_data(self):
        q = Quote.objects.filter(text__startswith="<p>Quote02")
        self.assertEqual(q.count(), 1,
                         "Couldn't find Quote02 after insertion")
        q = q[0]

        quotepage = self.getPage('show/%s/' % q.id)
        self.assertIn(q.text, quotepage)
        self.assertNotIn("author_notes", quotepage)
        self.assertNotIn("work_notes", quotepage)
        self.assertNotIn("tag_link", quotepage)

        workpage = self.getPage('work/%s/' % q.work.id)
        self.assertIn(q.text, workpage)
        self.assertNotIn("author_notes", quotepage)
        self.assertNotIn("work_notes", quotepage)
        self.assertNotIn("tag_link", quotepage)

        authorpage = self.getPage('author/%s/' % q.work.author.id)
        self.assertIn(q.text, authorpage)
        self.assertNotIn("author_notes", authorpage)
        self.assertNotIn("work_notes", authorpage)
        self.assertNotIn("tag_link", authorpage)
        self.assertNotIn('Quote01, two tags', authorpage)

    def test_view_author_notes_once(self):
        # check that on the per-author view, the author notes aren't display
        # for every quote
        a = Author.objects.filter(name="Author with notes")
        self.assertEqual(a.count(), 1)
        a = a[0]

        authorpage = self.getPage('author/%s/' % a.id)
        self.assertEqual(authorpage.count("Some notes for the author"), 1)
